#!/usr/bin/perl

#Copyright (C) 2006-2008 Keio University
#(Kris Popendorf) <comp@bio.keio.ac.jp> (2006)
#
#This file is part of Murasaki.
#
#Murasaki is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#Murasaki is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Murasaki.  If not, see <http://www.gnu.org/licenses/>.

require Bio::SeqFeature::Generic;
use Data::Dump qw{dump};
use File::Basename;

our %common_overrides;

sub writeOut {
  my $filename=shift;
  open(OF,">$filename") or die "Ack! Couldn't open $filename for writing!\n";
  print OF @_;
}

sub readableTime {
  my $t=shift;
  my $str="";
  my %r = (
  d => int($t/(60*60*24)),
  h => int($t%(60*60*24)/60/60),
  m => int($t%(60*60)/60),
  s => $t % 60);
  join(" ",map { "$r{$_}$_" } grep { $r{$_} } qw{d h m s});
}

sub parseHumanTime {
  my $total;
  my %r=( d => 24*60*60,
	  day => 24*60*60,
	  days => 24*60*60,
	  hour => 60*60,
	  hours => 60*60,
	  h => 60*60,
	  'minute' => 60,
	  'minutes' => 60,
	  'm' => 60,
	  'second' => 1,
	  'seconds' => 1,
	  's' => 1,
      );
  my $line=pop;
  my $tre=join("|",keys(%r));
  if($line=~m/\d+($tre)/){
    foreach (split(/\s+/,$line)){
      m/(\d+\.?\d*)(\w)/ or warn "Not a time bit: $_";
      warn "Unknown time unit: $2" unless exists($r{$2});
      $total+=$1*$r{$2};
    }
  }else{
    while($line=~m/(\d+\.?\d*) ($tre)/g){
      $total+=$1*$r{$2};
    }
  }
  return $total;
}

sub runLog {
  my ($file,$cmd)=@_;
  open(OF,">$file");
  open(IF,"$cmd|");
  while(<IF>){
#    print $_;
    print OF $_;
  }
  close OF;
  close IF;
}

sub mkFile {
  ($src,$dst,$genomes)=@_;
  $tpl=`cat $src`;
  $tpl=~s!{{GENOMES}}!$genomes!smg;
  $tpl=~s!{{ALIGNMENT}}!$output_path/$alignment!smg;
  $tpl=~s!{{CMD}}!$cmd!smg;
  $tpl=~s!{{MYDIR}}!$mydir!smg;
  open(DST,">$dst");
  print DST $tpl;
  close(DST);
}

sub readCds {
  my $file=shift;
  open(my $infh,"<",$file);
  my $offset=shift;
  $offset=0 unless $offset;
  my @features;
  while(<$infh>){
    my @a=split(/\s/,$_);
    unless($#a>2){
      print "Couldn't parse line $file $.: $_\n";
      next;
    }
    push(@features,new Bio::SeqFeature::Generic
	 (-primary => CDS,
	  -tag => {($a[0] ? (gene => $a[0]):()), ($a[4] ? (locus_tag=>$a[4]):())},
	  -start => $offset+$a[1],
	  -end => $offset+$a[2],
	  -strand => $a[3])
	);
  }
  close($infh);
  return @features;
}

sub readHacchyCds {
  #for seq=0 or 1. haccy gene_list contains coords for both
  my ($file,$seq)=(shift,shift);
  die "Gene_list not found: $file\n" unless -e $file;
  open($infh,"<",$file);
  my @features;
  while(<$infh>){
    #              1         2      3          4      5                6           7              8
#    next unless m/(\d+)\s+\((\d+),\s(\d+)\)\s+\((\d+),\s(\d+)\)\s+length=(\d+)\s+rate=(\d+)\s+strand=([+-])/;
    my @t=split(/[^0-9.+-]+/);
    my ($name,$start,$stop,$length,$rate,$strand)=($t[0],$t[1],$t[2],$t[5],$t[6],1);
    ($start,$stop)=($t[3],$t[4]) if $seq>0;
    $strand=-1 if $t[7] eq '-';

    push(@features,new Bio::SeqFeature::Generic
	 (-primary => CDS,
	  -tag => {gene => $name,
		   rate => $rate,
		  },
	  -start => $start,
	  -end => $stop,
	  -strand => $strand)
	);
  }
  print "Done with $file (sequence $seq)\n";
  close($infh);
  return @features;
}


sub writeCds {
  my $outf=shift;
  my ($gene_src,$locus_src)=("gene","locus_tag");
  $locus_src=$common_overrides{locus} if $common_overrides{locus};
  $gene_src=$common_overrides{gene} if $common_overrides{gene};
  open(my $outfh,">",$outf) or die "Couldn't open $outf for writing";
  foreach(@_){
    my ($name,$locus);
    $name=join(";",$_->get_tag_values($gene_src)) if $_->has_tag($gene_src);
    $name=join(";",$_->get_tag_values('gene')) if $_->has_tag('gene') && !$name;
    $name=join(";",$_->get_tag_values('protein_id')) if $_->has_tag('protein_id') && !$name;
    $name=join(";",$_->get_tag_values('product')) if $_->has_tag('product') && !$name;
    $name=$common_overrides{setName}  if exists $common_overrides{setName};
    $locus=join(";",$_->get_tag_values($locus_src)) if $_->has_tag($locus_src);
    if($common_overrides{extract} and $locus){
      my $pat=$common_overrides{extract};
      $locus=~m/$pat/;
      $locus=$1 if $1;
    }
    if($common_overrides{prefix} and $locus){ #I'm looking at you EcZ....
      $locus=$common_overrides{prefix}.$locus;
    }
    if($common_overrides{autoenum}){ #Cgl tRNA. no locus names
      our $lastLocusId;
      $locus.=$common_overrides{autoenum}.++$lastLocusId;
    }
    ($name,$locus)=map { $_ ? $_:"" } ($name,$locus);
    foreach($name,$locus){ $_=~s/\s/_/g };
    print $outfh join("\t",$name,
		      $_->location->start,
		      $_->location->end,
		      $_->location->strand,
		      $locus),"\n";
  }
  close($outfh);
}

sub repSuffix {
  my ($name,$path,$suffix) = fileparse($_[0], qr{\.[^.]*});
  return $path.$name.$_[1];
}


1;
